// Simple Large LED display clock project.
// By YH
// version 1.00
// Using Sure Electronics DP1311 LED matrix board

#define CS1  PORTD.RD0
#define CLK  PORTD.RD1
#define DATA PORTD.RD2

#define RC_MASTER_MODE 0b000110000
#define SYS_DIS        0b000000000
#define SYS_EN         0b000000010
#define LED_OFF        0b000000100
#define LED_ON         0b000000110
#define N_MOS_COM8     0b001000000
#define PWM_16         0b101011110
#define BLINK_OFF      0b000010000
#define BLINK_ON       0b000010010

#define SEMICOLON      10
#define LATIN_C        11
#define DEGREE_SYMBOL  12

// Initialization Functions here
void init();

// Generic HT1632 Functions here
void HT1632_Init();
void HT1632_Command(unsigned int command);
void HT1632_SendData(unsigned char address, unsigned char inputData);
void HT1632_WriteNum(unsigned char col, unsigned char num);
void frameBufferClear();
void HT1632_Clear();
void HT1632_DrawFrameBuffer();

// Generic DS1307 Functions here
unsigned char BCD2UpperInt(unsigned char bcd);
unsigned char BCD2LowerInt(unsigned char bcd);
unsigned char byte2BCD(unsigned char inputByte);
unsigned char BCD2Int(unsigned char bcd);
void write_ds1307(unsigned short address,unsigned short w_data);
unsigned short read_ds1307(unsigned short address);

// Generic DS18B20 Functions here (uses MikroC One-Wire Library)
void readTemperatureDS18B20();

// Variables here
unsigned char minute = 0;               // BCD values retrieved from DS1307
unsigned char hour = 0;                 //
unsigned char minute_temp = 0;          // Temporary BCD values for time setting etc...
unsigned char hour_temp = 0;            //
unsigned char temp_timer = 0;           //
unsigned char time[4] = { 0 };          // Time Digits for the display
unsigned char tempByte = 0;             // Temporary registers for conversion etc...
unsigned char tempTens = 0;             //
unsigned char tempOnes = 0;             //

unsigned char frameBuffer[32] = { 0 }; // Frame Buffer (32 x 8bits)
unsigned char scratchPadTemp[2] = { 0 };

// Font array for the LED Matrix
unsigned char font5x7[] = {

0x3E, 0x45, 0x49, 0x51, 0x3E,// 0
0x00, 0x40, 0x7F, 0x42, 0x00,// 1
0x46, 0x49, 0x51, 0x61, 0x42,// 2
0x31, 0x4B, 0x45, 0x41, 0x21,// 3
0x10, 0x7F, 0x12, 0x14, 0x18,// 4
0x39, 0x45, 0x45, 0x45, 0x27,// 5
0x30, 0x49, 0x49, 0x4A, 0x3C,// 6
0x03, 0x05, 0x09, 0x71, 0x01,// 7
0x36, 0x49, 0x49, 0x49, 0x36,// 8
0x1E, 0x29, 0x49, 0x49, 0x06,// 9
0x00, 0x00, 0x36, 0x36, 0x00,// :
0x22, 0x41, 0x41, 0x41, 0x3E,// C
0x00, 0x07, 0x05, 0x07, 0x00 // degree symbol

};

void main() {

unsigned char isTimeSet = 0;    // is time set yet? (1 = set, 0 = not set)

init();

HT1632_Init();

delay_ms(250);

isTimeSet = read_ds1307(0x08);  // Register must be read twice - must look into it. Give some time to warm up, perhaps?
isTimeSet = read_ds1307(0x08);

if (isTimeSet == 1) {// Get data from RTC while battery is installed.

        hour = read_ds1307(2);
        minute = read_ds1307(1);
        HT1632_Command(BLINK_OFF);

}
else {// if RTC battery removed, reset time to default values.

        write_ds1307(0, 0b10000000);    //seconds (last MSB is clock halt)
        write_ds1307(1, byte2BCD(00));  //minutes
        write_ds1307(2, byte2BCD(00));  //hours
        write_ds1307(0, 0b00000000);    //seconds (last MSB is clock halt)
        HT1632_Command(BLINK_ON);

}

frameBufferClear();
HT1632_Clear();

readTemperatureDS18B20();

delay_ms(2000);                // wait for the DS18B20 to fully power up first.

while(1)
{
frameBufferClear();

hour = read_ds1307(2);
minute = read_ds1307(1);

if(PORTB.RB0 == 0)
{
   delay_ms(5);
   hour_temp = BCD2Int(hour);
   hour_temp++;
   if(hour_temp > 23)
      hour_temp = 0;
   write_ds1307(2, byte2BCD(hour_temp));
   write_ds1307(0x08, 0x01);                  // time is set already
   HT1632_Command(BLINK_OFF);
}

if(PORTB.RB1 == 0)
{
   delay_ms(5);
   minute_temp = BCD2Int(minute);
   minute_temp++;
   if(minute_temp > 59)
      minute_temp = 0;
   write_ds1307(1, byte2BCD(minute_temp));
   write_ds1307(0x08, 0x01);                  // time is set already
   HT1632_Command(BLINK_OFF);
}


if(PORTB.RB2 == 0)
{
   delay_ms(5);
   frameBufferClear();
   while(temp_timer < 5)
   {
   delay_ms(1000);
   temp_timer++;
   HT1632_WriteNum(27-4, tempTens);
   HT1632_WriteNum(21-4, tempOnes);
   HT1632_WriteNum(15-4, DEGREE_SYMBOL);
   HT1632_WriteNum(9-4,  LATIN_C);
   HT1632_DrawFrameBuffer();
   }
   temp_timer = 0;
   frameBufferClear();
}

time[0] = BCD2UpperInt(hour);
time[1] = BCD2LowerInt(hour);
time[2] = BCD2UpperInt(minute);
time[3] = BCD2LowerInt(minute);

readTemperatureDS18B20();

HT1632_WriteNum(27-2, time[0]);

HT1632_WriteNum(21-2, time[1]);

HT1632_WriteNum(15-2, 10);

HT1632_WriteNum(9-1, time[2]);

HT1632_WriteNum(3-1, time[3]);

HT1632_DrawFrameBuffer();

delay_ms(250);

}

}

void init()
{
ADCON0.ADON = 0;           // shut off ADC
ADCON1.PCFG3 = 0;          // all pins Digital
ADCON1.PCFG2 = 1;
ADCON1.PCFG1 = 1;
ADCON1.PCFG0 = 0;

TRISC = 0x00;
TRISD = 0x00;
TRISB.RB0 = 1;
TRISB.RB1 = 1;
TRISB.RB2 = 1;

OPTION_REG.INTEDG = 0; // falling edge interrupt
INTCON.INTE = 0;       // disable external interrupt at RB0
INTCON.PEIE = 0;
INTCON.GIE = 0;        // disable all interrupts here
INTCON.INTF = 0;       // clear external interrupt flag


I2C1_Init(100000);
I2C1_Start();

}

void HT1632_Command(unsigned int command)
{

unsigned char i;
unsigned int j, k, regAMSB;

CS1 = 1;
delay_us(1);

j = command << 4;

regAMSB = 0b1000000000000000 | j;

CS1 = 0;
delay_us(1);

for(i=0; i<16; i++)
{
   DATA = (regAMSB >> 15) & 0x0001;
   regAMSB = regAMSB << 1;
   CLK = 0;
   delay_us(1);
   CLK = 1;
   delay_us(1);
}

CS1 = 1;
delay_us(1);

}

void HT1632_SendData(unsigned char address, unsigned char inputData)
{

unsigned char i;
unsigned int j, l, regAMSB;

CS1 = 1;
delay_us(1);

j = (address & 0x7F) << 6;
l = (inputData & 0x000f) << 2;
regAMSB = 0b1010000000000000 | j | l;

regAMSB = regAMSB >> 2;

CS1 = 0;
delay_us(1);

for(i=0; i<14; i++)
{
   DATA = (regAMSB >> 13) & 0x0001;
   regAMSB = regAMSB << 1;
   CLK = 0;
   delay_us(1);
   CLK = 1;
   delay_us(1);
}

CS1 = 1;
delay_us(1);

}

void HT1632_WriteNum(unsigned char col, unsigned char num)
{

 unsigned char i;
 
 for(i = 0; i < 5; i++)
   frameBuffer[col+i] |= font5x7[num*5 + i];

}

void HT1632_DrawFrameBuffer()
{

unsigned char i, j;

for(i = 0; i < 32; i++)
     HT1632_SendData(2*i,   (frameBuffer[i] >> 4) & 0x0f);

for(i = 0; i < 32; i++)
     HT1632_SendData(2*i+1, frameBuffer[i] & 0x0f);

}

void HT1632_Init()
{

HT1632_Command(SYS_EN);
HT1632_Command(LED_ON);
HT1632_Command(RC_MASTER_MODE);
HT1632_Command(N_MOS_COM8);
HT1632_Command(PWM_16);
HT1632_Command(BLINK_OFF);

}

void frameBufferClear()
{
unsigned char i = 0;

for(i = 0; i < 32; i++)
     frameBuffer[i] = 0x00;

}

void HT1632_Clear()
{

unsigned char i = 0;

for(i = 0; i < 64; i++)
   HT1632_SendData(i, 0x00);

}

void readTemperatureDS18B20()
{

Ow_Reset(&PORTA, 0);
Ow_Write(&PORTA, 0, 0xCC);                   // Issue command SKIP_ROM
Ow_Write(&PORTA, 0, 0x44);                   // Issue command CONVERT_T

delay_us(120);

Ow_Reset(&PORTA, 0);
Ow_Write(&PORTA, 0, 0xCC);                   // Issue command SKIP_ROM
Ow_Write(&PORTA, 0, 0xBE);                   // Issue command READ_SCRATCHPAD

scratchPadTemp[0] = Ow_Read(&PORTA, 0);      // LSB
scratchPadTemp[1] = Ow_Read(&PORTA, 0);      // MSB

scratchpadTemp[0] = 0b11110000 & scratchpadTemp[0];
scratchpadTemp[1] = 0b00000111 & scratchpadTemp[1];

tempByte = (scratchpadTemp[1] << 4) | (scratchpadTemp[0] >> 4);

tempTens = tempByte / 10;
tempOnes = tempByte % 10;

}


unsigned char BCD2UpperInt(unsigned char bcd) {
    return (bcd & 0xf0) >> 4;
}

unsigned char BCD2LowerInt(unsigned char bcd) {
    return ((bcd & 0x0F));
}

unsigned char byte2BCD(unsigned char inputByte) {
    unsigned char tens, ones, BCDresult;

    tens = inputByte / 10;
    ones = inputByte % 10;

    BCDresult = ((tens & 0x0f) << 4) | (ones & 0x0f);

    return BCDresult;
}

unsigned char BCD2Int(unsigned char bcd) {
    return ((BCD2UpperInt(bcd) * 10) + BCD2LowerInt(bcd));
}

void write_ds1307(unsigned short address,unsigned short w_data)
{
I2C1_Start(); // issue I2C start signal
//address 0x68 followed by direction bit (0 for write, 1 for read) 0x68 followed by 0 --> 0xD0
I2C1_Wr(0xD0); // send byte via I2C (device address + W)
I2C1_Wr(address); // send byte (address of DS1307 location)
I2C1_Wr(w_data); // send data (data to be written)
I2C1_Stop(); // issue I2C stop signal
}

unsigned short read_ds1307(unsigned short address)
{

unsigned short readData;
I2C1_Start();
I2C1_Wr(0xd0); //address 0x68 followed by direction bit (0 for write, 1 for read) 0x68 followed by 0 --> 0xD0
I2C1_Wr(address);
I2C1_Repeated_Start();
I2C1_Wr(0xd1); //0x68 followed by 1 --> 0xD1
readData=I2C1_Rd(0);
I2C1_Stop();
return readData;
}